
# Automated version of below?
# Check for factors in formula (after generalize simcf to factors)
# Check for compositional data by considering every combination of X's and checking for sum to common constraint within tolerance
# report guesses if verbose

pcp.glm <- function(res, y, type="model") { # other types:  null, improve
  
  pcp <- mean(round(predict(res,type="response"))==y)
  pcpNull <- max(mean(y), mean(1-y))
  pcpImprove <- (pcp-pcpNull)/(1-pcpNull)

  if (type=="pcp")
    return(pcp)
  if (type=="null")
    return(pcpNull)
  if (type=="improve")
    return(pcpImprove)
}

pcp.oprobit <- function(x, y, b, constant=1, ncat=3, type="model") { # other types:  null, improve

  require(simcf)
  b <- matrix(b,nrow=100,ncol=length(b),byrow=TRUE)
  simy <- oprobitsimev(x, b, constant=constant, cat=ncat)

  cats <- sort(unique(y))
  
  predcatN <- cats[rev(order(table(y)))][1]

  n <- length(y)
  pcp <- pcpNull <- predcatM <- rep(NA,n)
  for (i in 1:n) {
    predcatM[i] <- cats[rev(order(simy$pe[i,]))][1]
    pcp[i] <- predcatM[i]==y[i]
    pcpNull[i] <- predcatN==y[i]
  }
  
  pcp <- mean(pcp)
  pcpNull <- mean(pcpNull)
  pcpImprove <- (pcp-pcpNull)/(1-pcpNull)
  
  if (type=="model")
    return(pcp)
  if (type=="null")
    return(pcpNull)
  if (type=="improve")
    return(pcpImprove)
  
}


concord.glm <- function(res, y, type="model") { # other types:  null, improve
  require(verification)
  if (type=="model") {
    yhat <- predict(res,type="response")
    concord <- roc.area(y, yhat)$A
  }
  if (type=="null") {    
    yhat <- rep(max(mean(y), mean(1-y)), length(y))
    concord <- roc.area(y, yhat)$A
  }
  if (type=="improve") {
    yhat <- predict(res,type="response")
    model <- roc.area(y, yhat)$A
    yhat <- rep( max(mean(y), mean(1-y)), length(y))
    null <- roc.area(y, yhat)$A
    concord <- (model-null)/(1-null)
  }
  concord
}


concord.oprobit <- function(x, y, b, constant=1, ncat=3, type="model") { # other types:  null, improve
  require(simcf)
  require(verification)
  b <- matrix(b,nrow=100,ncol=length(b),byrow=TRUE)
  simy <- oprobitsimev(x, b, constant=constant, cat=ncat)
  cats <- sort(unique(y))

  if (type=="model") {
    model <- rep(NA,ncat)
    for (j in 1:ncat) {
      yhat <- simy$pe[,j]
      model[j] <- roc.area(as.numeric(y==cats[j]), yhat)$A
    }
    concord <- mean(model) 
  }

  if (type=="null") {
    null <- rep(NA,ncat)
    for (j in 1:ncat) {
      probs <- rep(mean(y==cats[j]), length(y))      
      null[j] <- roc.area(as.numeric(y==cats[j]), probs)$A
    }
    concord <- mean(null) 
  }
  
  if (type=="improve") {
    improve <- rep(NA,ncat)
    for (j in 1:ncat) {
      probs <- rep(mean(y==cats[j]), length(y))
      null <- roc.area(as.numeric(y==cats[j]), probs)$A
      yhat <- simy$pe[,j]
      model <- roc.area(as.numeric(y==cats[j]), yhat)$A
      improve[j] <- (model-null)/(1-null)
    }
    concord <- mean(improve) 
  }
  
  concord
}


setupCF <- function(x,xvar,model,fd=FALSE,extra=NULL) {

  k <- length(xvar)
  newdata <- NULL
  newpredata <- NULL
  scenarios <- NULL


  # Create global means and sd's
  meandata <- apply(x,2,mean)
  sddata <- apply(x,2,sd)

  ######################
  # Change from here

  # Special treatment for logged averages
  scenvar <- "logpop"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    meanLogpop <- log(mean(exp(x[,curxrow])))
    sdLogpop <- log(sd(exp(x[,curxrow])))
    meanplussdLogpop <- log(mean(exp(x[,curxrow])) + sd(exp(x[,curxrow])))
    meandata[curxrow] <- meanLogpop
    sddata[curxrow] <- sdLogpop
  }
  

  # Scenarios 
  
  scenvar <- "DemGov"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- 1
    newprescen <- meandata
    newprescen[curxrow] <- 0
    
    if (any(xvar=="DemLeg")) {
      currow <- (1:k)[xvar=="DemLeg"]
      newscen[currow] <- 1
      newprescen[currow] <-0
    

      scenname <- "Dem Gov + Dem Leg"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
      
      
      
      curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      newscen[curxrow] <- 0
      newprescen <- meandata
      newprescen[curxrow] <- 0
      
      currow <- (1:k)[xvar=="DemLeg"]
      newscen[currow] <- 0
      newprescen[currow] <-0
    

      scenname <- "Rep Gov + Rep Leg"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
      


      curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      newscen[curxrow] <- 1
      newprescen <- meandata
      newprescen[curxrow] <- 0
    
      currow <- (1:k)[xvar=="DemLeg"]
      newscen[currow] <- 0
      newprescen[currow] <-0
    

      scenname <- "Dem Gov + Rep Leg"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
      
      curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      newscen[curxrow] <- 0
      newprescen <- meandata
      newprescen[curxrow] <- 0
      
      
      currow <- (1:k)[xvar=="DemLeg"]
      newscen[currow] <- 1
      newprescen[currow] <-0
    

      scenname <- "Rep Gov + Dem Leg"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
    } else {
      scenname <- "Democratic Governor"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)


      curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      newscen[curxrow] <- 0
      newprescen <- meandata
      newprescen[curxrow] <- 1
      scenname <- "Republican Governor"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
    
    } 
  }

  
  # Factors of redistribution

  
  scenvar <- "redistFA1"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "Young, Not Elderly (Factor 1)"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "redistFA2"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "Poor, Black (Factor 2)"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "redistFA3"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "Non-Hispanic & Low Income (Factor 3)"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  
  

  # Budget Shares of Total Budget
  scenvar <- "PublicHealthS"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="HospitalsS")) {
      currow <- (1:k)[xvar=="HospitalsS"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="MedicaidS")) {
      currow <- (1:k)[xvar=="MedicaidS"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Public Health Share +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "PublicHealthSL1"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="HospitalsSL1")) {
      currow <- (1:k)[xvar=="HospitalsSL1"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="MedicaidSL1")) {
      currow <- (1:k)[xvar=="MedicaidSL1"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Public Health Share L1 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }
  
  scenvar <- "PublicHealthSL5"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="HospitalsSL5")) {
      currow <- (1:k)[xvar=="HospitalsSL5"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="MedicaidSL5")) {
      currow <- (1:k)[xvar=="MedicaidSL5"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Public Health Share L5 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }


   scenvar <- "PublicHealthSL10"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="HospitalsSL10")) {
      currow <- (1:k)[xvar=="HospitalsSL10"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="MedicaidSL10")) {
      currow <- (1:k)[xvar=="MedicaidSL10"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Public Health Share L10 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }


   scenvar <- "HospitalsS"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="PublicHealthS")) {
      currow <- (1:k)[xvar=="PublicHealthS"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="MedicaidS")) {
      currow <- (1:k)[xvar=="MedicaidS"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Hospital Share +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "HospitalsSL1"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="PublicHealthSL1")) {
      currow <- (1:k)[xvar=="PublicHealthSL1"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="MedicaidSL1")) {
      currow <- (1:k)[xvar=="MedicaidSL1"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Hospital Share L1 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }
  
  scenvar <- "HospitalsSL5"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="PublicHealthSL5")) {
      currow <- (1:k)[xvar=="PublicHealthSL5"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="MedicaidSL5")) {
      currow <- (1:k)[xvar=="MedicaidSL5"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Hospital Share L5 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }


   scenvar <- "HospitalsSL10"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="PublicHealthSL10")) {
      currow <- (1:k)[xvar=="PublicHealthSL10"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="MedicaidSL10")) {
      currow <- (1:k)[xvar=="MedicaidSL10"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Hospital Share L10 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }


  

   scenvar <- "MedicaidS"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="PublicHealthS")) {
      currow <- (1:k)[xvar=="PublicHealthS"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="HospitalsS")) {
      currow <- (1:k)[xvar=="HospitalsS"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Medicaid Share +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "MedicaidSL1"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="PublicHealthSL1")) {
      currow <- (1:k)[xvar=="PublicHealthSL1"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="HospitalsSL1")) {
      currow <- (1:k)[xvar=="HospitalsSL1"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Medicaid Share L1 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }
  
  scenvar <- "MedicaidSL5"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="PublicHealthSL5")) {
      currow <- (1:k)[xvar=="PublicHealthSL5"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="HospitalsSL5")) {
      currow <- (1:k)[xvar=="HospitalsSL5"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Medicaid Share L5 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }


   scenvar <- "MedicaidSL10"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="PublicHealthSL10")) {
      currow <- (1:k)[xvar=="PublicHealthSL10"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="HospitalsSL10")) {
      currow <- (1:k)[xvar=="HospitalsSL10"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Medicaid Share L10 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }


  
  # Budget Shares of Health Budget
  
  scenvar <- "PublicHealthHS"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="HospitalsHS")) {
      currow <- (1:k)[xvar=="HospitalsHS"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="MedicaidHS")) {
      currow <- (1:k)[xvar=="MedicaidHS"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Public Health HShare +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "PublicHealthHSL1"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="HospitalsHSL1")) {
      currow <- (1:k)[xvar=="HospitalsHSL1"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="MedicaidHSL1")) {
      currow <- (1:k)[xvar=="MedicaidHSL1"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Public Health HShare L1 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }
  
  scenvar <- "PublicHealthHSL5"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="HospitalsHSL5")) {
      currow <- (1:k)[xvar=="HospitalsHSL5"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="MedicaidHSL5")) {
      currow <- (1:k)[xvar=="MedicaidHSL5"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Public Health HShare L5 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }


   scenvar <- "PublicHealthHSL10"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="HospitalsHSL10")) {
      currow <- (1:k)[xvar=="HospitalsHSL10"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="MedicaidHSL10")) {
      currow <- (1:k)[xvar=="MedicaidHSL10"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Public Health HShare L10 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }


   scenvar <- "HospitalsHS"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="PublicHealthHS")) {
      currow <- (1:k)[xvar=="PublicHealthHS"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="MedicaidHS")) {
      currow <- (1:k)[xvar=="MedicaidHS"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Hospital HShare +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "HospitalsHSL1"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="PublicHealthHSL1")) {
      currow <- (1:k)[xvar=="PublicHealthHSL1"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="MedicaidHSL1")) {
      currow <- (1:k)[xvar=="MedicaidHSL1"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Hospital HShare L1 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }
  
  scenvar <- "HospitalsHSL5"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="PublicHealthHSL5")) {
      currow <- (1:k)[xvar=="PublicHealthHSL5"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="MedicaidHSL5")) {
      currow <- (1:k)[xvar=="MedicaidHSL5"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Hospital HShare L5 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }


   scenvar <- "HospitalsHSL10"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="PublicHealthHSL10")) {
      currow <- (1:k)[xvar=="PublicHealthHSL10"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="MedicaidHSL10")) {
      currow <- (1:k)[xvar=="MedicaidHSL10"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Hospital HShare L10 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "MedicaidHS"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="PublicHealthHS")) {
      currow <- (1:k)[xvar=="PublicHealthHS"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="HospitalsHS")) {
      currow <- (1:k)[xvar=="HospitalsHS"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Medicaid HShare +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "MedicaidHSL1"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="PublicHealthHSL1")) {
      currow <- (1:k)[xvar=="PublicHealthHSL1"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="HospitalsHSL1")) {
      currow <- (1:k)[xvar=="HospitalsHSL1"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Medicaid HShare L1 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }
  
  scenvar <- "MedicaidHSL5"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="PublicHealthHSL5")) {
      currow <- (1:k)[xvar=="PublicHealthHSL5"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="HospitalsHSL5")) {
      currow <- (1:k)[xvar=="HospitalsHSL5"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Medicaid HShare L5 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }


  scenvar <- "MedicaidHSL10"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="PublicHealthHSL10")) {
      currow <- (1:k)[xvar=="PublicHealthHSL10"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    if (any(xvar=="HospitalsHSL10")) {
      currow <- (1:k)[xvar=="HospitalsHSL10"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "Medicaid HShare L10 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "AllHealthPC"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "PC Health Spending +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "AllHealthPCL1"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "PC Health Spending L1 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "AllHealthPCL5"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "PC Health Spending L5 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "AllHealthPCL10"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "PC Health Spending L10 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "logAllHealth"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "log Total Health Spending +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "logAllHealthL1"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "log Total Health Spending L1 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "logAllHealthL5"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "log Total Health Spending L5 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "logAllHealthL10"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "log Total Health Spending L10 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "AllHealth"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "Total Health Spending +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "AllHealthL1"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "Total Health Spending L1 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "AllHealthL5"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "Total Health Spending L5 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "AllHealthL10"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "Total Health Spending L10 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }


  
  scenvar <- "AllExpendPC"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "PC Spending +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "AllExpendPCL1"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "PC Spending L1 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "AllExpendPCL5"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "PC Spending L5 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "AllExpendPCL10"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "PC Spending L10 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "logAllExpend"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "log Total Spending +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "logAllExpendL1"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "log Total Spending L1 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "logAllExpendL5"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "log Total Spending L5 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "logAllExpendL10"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "log Total Spending L10 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "AllExpend"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "Total Spending +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "AllExpendL1"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "Total Spending L1 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "AllExpendL5"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "Total Spending L5 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "AllExpendL10"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- exp(meandata[curxrow] + sddata[curxrow])
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "Total Spending L10 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }


  scenvar <- "logpop"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meanplussdLogpop
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "Population +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  
  
  scenvar <- "South"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- 1
    newprescen <- meandata
    #newprescen[curxrow] <- 0
    if (any(xvar=="Northeast")) {
      currow <- (1:k)[xvar=="Northeast"]
      newscen[currow] <- 0
      #newprescen[currow] <- 0
    }
    if (any(xvar=="West")) {
      currow <- (1:k)[xvar=="West"]
      newscen[currow] <- 0
      #newprescen[currow] <- 0
    }
    scenname <- "South"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  

    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- 0
    newprescen <- meandata
    #newprescen[curxrow] <- 0
    if (any(xvar=="Northeast")) {
      currow <- (1:k)[xvar=="Northeast"]
      newscen[currow] <- 1
      #newprescen[currow] <- 0
    }
    if (any(xvar=="West")) {
      currow <- (1:k)[xvar=="West"]
      newscen[currow] <- 0
      #newprescen[currow] <- 0
    }
    scenname <- "Northeast"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- 0
    newprescen <- meandata
    #newprescen[curxrow] <- 0
    if (any(xvar=="Northeast")) {
      currow <- (1:k)[xvar=="Northeast"]
      newscen[currow] <- 0
      #newprescen[currow] <- 0
    }
    if (any(xvar=="West")) {
      currow <- (1:k)[xvar=="West"]
      newscen[currow] <- 1
      #newprescen[currow] <- 0
    }
    scenname <- "West"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  

    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- 0
    newprescen <- meandata
    #newprescen[curxrow] <- 0
    if (any(xvar=="Northeast")) {
      currow <- (1:k)[xvar=="Northeast"]
      newscen[currow] <- 0
      #newprescen[currow] <- 0
    }
    if (any(xvar=="West")) {
      currow <- (1:k)[xvar=="West"]
      newscen[currow] <- 0
      #newprescen[currow] <- 0
    }
    scenname <- "Midwest"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }
  
  scenvar <- "unidem"
  if (any(xvar==scenvar)) {
    if (fd) {
      curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      newscen[curxrow] <- 1
      newprescen <- meandata
      newprescen[curxrow] <- 0
      if (any(xvar=="unirep")) {
        currow <- (1:k)[xvar=="unirep"]
        newscen[currow] <- 0
        newprescen[currow] <- 0
      }
      scenname <- "Divided -> Democratic Goverment"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
      
      curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      newscen[curxrow] <- 0
      newprescen <- meandata
      newprescen[curxrow] <- 0
      if (any(xvar=="unirep")) {
        currow <- (1:k)[xvar=="unirep"]
        newscen[currow] <- 1
        newprescen[currow] <- 0
      }
      scenname <- "Divided -> Republican Goverment"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
      
      curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      newscen[curxrow] <- 1
      newprescen <- meandata
      newprescen[curxrow] <- 0
      if (any(xvar=="unirep")) {
        currow <- (1:k)[xvar=="unirep"]
        newscen[currow] <- 0
        newprescen[currow] <- 1
      }
      scenname <- "Republican -> Democratic Goverment"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
      
      curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      newscen[curxrow] <- 0
      newprescen <- meandata
      newprescen[curxrow] <- 1
      if (any(xvar=="unirep")) {
        currow <- (1:k)[xvar=="unirep"]
        newscen[currow] <- 1
        newprescen[currow] <- 0
      }
      scenname <- "Democratic -> Republican Goverment"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname) 
    } else {  # expected levels only:

      curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      newscen[curxrow] <- 1
      newprescen <- meandata
      newprescen[curxrow] <- 0
      if (any(xvar=="unirep")) {
        currow <- (1:k)[xvar=="unirep"]
        newscen[currow] <- 0
        newprescen[currow] <- 0
      }
      scenname <- "Democratic Goverment"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)

      
      curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      newscen[curxrow] <- 0
      newprescen <- meandata
      newprescen[curxrow] <- 0
      if (any(xvar=="unirep")) {
        currow <- (1:k)[xvar=="unirep"]
        newscen[currow] <- 1
        newprescen[currow] <- 0
      }
      scenname <- "Republican Goverment"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)


      curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      newscen[curxrow] <- 0
      newprescen <- meandata
      newprescen[curxrow] <- 0
      if (any(xvar=="unirep")) {
        currow <- (1:k)[xvar=="unirep"]
        newscen[currow] <- 0
        newprescen[currow] <- 0
      }
      scenname <- "Divided Goverment"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
    }
  } else {
    scenvar <- "unirep"
    if (any(xvar==scenvar)) {
      curxrow <- (1:k)[xvar==scenvar]
      newscen <- meandata
      newscen[curxrow] <- 1
      newprescen <- meandata
      newprescen[curxrow] <- 0
      scenname <- "Republican Goverment"
      newdata <- rbind(newdata,newscen)
      newpredata <- rbind(newpredata,newprescen)
      scenarios <- c(scenarios,scenname)
    }
  }
  
  scenvar <- "pctover65"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    if (any(xvar=="pctunder19")) {
      currow <- (1:k)[xvar=="pctunder19"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "% Population over 65 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }
  
  
  scenvar <- "pctunder19"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
     if (any(xvar=="pctover65")) {
      currow <- (1:k)[xvar=="pctover65"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "% Population under 19 +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }
  
  scenvar <- "PctWelfareFed"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "Federal Social Welfare Aid +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }
  
  scenvar <- "PctHealthFed"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "Federal Health Aid +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }
  
  scenvar <- "RealPctChBudgetPC"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "State Budget Growth per cap +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }
  
  scenvar <- "pcthisp"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
     if (any(xvar=="pctblack")) {
      currow <- (1:k)[xvar=="pctblack"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "% Hispanic Population +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }
  
  scenvar <- "pctblack"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
     if (any(xvar=="pcthisp")) {
      currow <- (1:k)[xvar=="pcthisp"]
      rpcfVal <- rpcf(c(meandata[curxrow], meandata[currow], (1-meandata[curxrow] - meandata[currow])), 1, sddata[curxrow]) 
      newscen[currow] <- rpcfVal[2]
      newprescen[currow] <- meandata[currow]
    }
    scenname <- "% Black Population +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }
  
  scenvar <- "povrate"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "Poverty Rate +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }
  
  scenvar <- "rpincpc"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "Income Per capita +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }
  
  scenvar <- "popdensity"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "Population Density +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }

  scenvar <- "Unemployment"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    newscen <- meandata
    newscen[curxrow] <- meandata[curxrow] + sddata[curxrow]
    newprescen <- meandata
    newprescen[curxrow] <- meandata[curxrow]
    scenname <- "Unemployment +1 sd"
    newdata <- rbind(newdata,newscen)
    newpredata <- rbind(newpredata,newprescen)
    scenarios <- c(scenarios,scenname)
  }


  # create means by 1990 and 2010.  add scenarios

  earlyMeandata <- apply(x[extra$Year==min(extra$Year),],2,mean)
  scenvar <- "logpop"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    earlyMeandata[curxrow] <- log(mean(exp(x[extra$Year==min(extra$Year),curxrow])))
  }
  newscen <- earlyMeandata
  newprescen <- meandata
  scenname <- paste("Average State in",min(extra$Year))
  newdata <- rbind(newdata,newscen)
  newpredata <- rbind(newpredata,newprescen)
  scenarios <- c(scenarios,scenname)

  lateMeandata <- apply(x[extra$Year==max(extra$Year),],2,mean)
  scenvar <- "logpop"
  if (any(xvar==scenvar)) {
    curxrow <- (1:k)[xvar==scenvar]
    lateMeandata[curxrow] <- log(mean(exp(x[extra$Year==max(extra$Year),curxrow])))
  }
  newscen <- lateMeandata
  newprescen <- meandata
  scenname <- paste("Average State in",max(extra$Year))
  newdata <- rbind(newdata,newscen)
  newpredata <- rbind(newpredata,newprescen)
  scenarios <- c(scenarios,scenname)

  # Change up to here
  ###########
  
  # Modify this
  newscen <- meandata
  newprescen <- meandata
  scenname <- "Average State"
  newdata <- rbind(newdata,newscen)
  newpredata <- rbind(newpredata,newprescen)
  scenarios <- c(scenarios,scenname)
  

  #newdata <- as.data.frame(newdata)
  #newpredata <- as.data.frame(newpredata)
  rownames(newdata) <- scenarios
  rownames(newpredata) <- scenarios
  
  result <- list(x=as.data.frame(newdata),xpre=as.data.frame(newpredata),scenarios=scenarios,model=model)
  class(result) <- "counterfactual"
  result
  }


firstdiffTable <- function(filename,fd,n,scen) {

  fd$lower <- t(fd$lower)
  fd$upper <- t(fd$upper)
  
  rankfd <- fd$rank
  pctfd <- -100*fd$rank/n
  lowerfd <- fd$lower
  upperfd <- fd$upper
  lowerpct <- -100*fd$lower/n
  upperpct <- -100*fd$upper/n

  result <- cbind(scen,rankfd,pctfd,lowerfd,upperfd,lowerpct,upperpct)
  result <- as.data.frame(result)
  names(result) <- c("case","x","px","lowerx1","lowerx2","upperx1","upperx2","lowerpx1","lowerpx2","upperpx1","upperpx2")

  write.csv(result,paste(filename,"csv",sep="."))

}



getxvar <- function(xstrings,imp=5) {
  X <- NULL
  for (i in 1:length(xstrings)) {
    nextx <- try(get(xstrings[i]),silent=TRUE)
    if (any(class(nextx)=="try-error")) {
      nextx <- try(get(paste(xstrings[i],"i",imp,sep="")),silent=TRUE)
      if (any(class(nextx)=="try-error")) {
        stop(paste("Variable ", xstrings[i], " not found",sep=""))
      }
    }
    X <- cbind(X,nextx)
  }
  colnames(X) <- xstrings
  X
}


# To do here:  DONE? Generate multiple plots for op, mnl.  based on dim of simy$pe


cfPlot <- function(simy, xscen, response=NULL, gof=NULL, fd=FALSE, log=FALSE, at=NULL, pch=16,
                   baseline=NULL, limits=NULL, entryheight=1/6, plotwidth=4, output=NULL, sortcol=1,
                   main=NULL) {
  require(tile)

  nplots <- ncol(as.matrix(simy$pe))

  if (length(pch)==1) pch <- matrix(pch, nrow=length(simy$pe[,1]), ncol=nplots)
  
  if (fd) {
    if (is.null(limits))
      limits <- c(-1,1)
  } else {
    if (is.null(limits))
      limits <- c(0,1)
  }
  
  pe <- as.matrix(simy$pe)
  lw <- as.matrix(simy$lower)
  up <- as.matrix(simy$upper)
  
  lw <- lw[order(pe[,sortcol]),,drop=FALSE]
  up <- up[order(pe[,sortcol]),,drop=FALSE]
  sc <- xscen$scenarios[order(pe[,sortcol])]
  pch <- pch[order(pe[,sortcol]),,drop=FALSE]
  pe <- pe[order(pe[,sortcol]),,drop=FALSE]
  
  if (fd) {
    last <- nrow(pe)
    pe <- pe[1:(last-1),,drop=FALSE]
    lw <- lw[1:(last-1),,drop=FALSE]
    up <- up[1:(last-1),,drop=FALSE]
    pch <- pch[1:(last-1),,drop=FALSE]
  }

  if (length(response)<nplots) response <- rep(response,nplots)
  if (any(class(limits)=="matrix")) {
    while (nrow(limits)<nplots) limits <- rbind(limits,limits)
  } else {
    limits <- matrix(limits, ncol=2, nrow=nplots, byrow=TRUE)
  }

  maintitle <- list()
  if (!is.null(main)) {
    maintitle$labels <- main
  }
  
  topaxistitle <- topaxis <- list()
  if (!is.null(baseline)) {
    if (length(baseline)<nplots) baseline <- rep(baseline,nplots)
     
    topaxis$add <- TRUE
    if (log) topaxis$log <- TRUE
    topaxis$cex <- 0.8
    
    for (i in 1:nplots) {
      axisRR <- mirrorRR(limits=limits[i,], reference=baseline[i], fd=FALSE)
      topaxis[[paste("at",i,sep="")]] <- axisRR$at
      topaxis[[paste("labels",i,sep="")]] <- axisRR$labels
      topaxistitle[[paste("labels",i,sep="")]] <- paste("Rel Risk(",response[i],")",sep="")
    }
    
    gridlines <- list(type="t")    

  } else {
    gridlines <- list(type="x")
  }     

  xaxis <- list(cex=.8)
  if (log) xaxis$log <- TRUE
  if (!is.null(at)) {
    if (any(class(at)=="matrix")) {
      while (nrow(at)<nplots) at <- rbind(at,at)
    } else {
      at <- matrix(at,nrow=nplots,ncol=length(at),byrow=TRUE)
    }
  }   
    
  xaxistitle <- list()
  for (i in 1:nplots) {
    xaxistitle[[paste("labels",i,sep="")]] <- paste("Pr(",response[i],")",sep="")
    if (!is.null(at)) xaxis[[paste("at",i,sep="")]] <- na.omit(at[i,])
  }

  if (!is.null(gof)) {
    undertitle <- list(labels=gof, fontface="italic", cex=0.8)
  } else {
    undertitle <- list()
  }

  if (!is.null(output)) {
    output=list(file=output)
  } else {
    output=list()
  }
  
  ctraces <- vertline <- vector("list",nplots)

  ctr <- 0
  for (i in 1:nplots) {
    ctr <- ctr + 1
    if (i==1) {labels <- sc} else {labels <- NULL}
    ctraces[[ctr]] <- ropeladder(x=pe[,i],
                                 lower=lw[,i],
                                 upper=up[,i],
                                 labels=labels,
                                 pch=pch[,i],
                                 entryheight=entryheight,
                                 plot=i)

    if (!is.null(baseline)) {
      vertline[[ctr]] <- linesTile(x=c(baseline[i],baseline[i]),
                                   y=c(0,1),lty="solid",plot=i)
    }
    
    #plotlabels <- c(plotlabels)
  }

  if (!is.null(baseline)) ctraces <- c(ctraces, vertline)
  
  tile(ctraces,
       limits=limits,
       #plottitle=plottitle,
       maintitle=maintitle,
       undertitle=undertitle,
       topaxistitle=topaxistitle,
       xaxis=xaxis,
       xaxistitle=xaxistitle,
       topaxis=topaxis,
       gridlines=gridlines,
       width=list(null=plotwidth),
       height=list(undertitle=0.2, xaxistitle=3, topaxistitle=3, maintitle=4),
       output=output
       )
  
  invisible(0)
}


# Test code for function below
#baseline <- 1
#limits <- matrix(c(0.00001, 0.0001, 0.001, 0.01, 0.1, 0.2, 0.5, 0.7, 0.8, 0.9, 0.95,
#                   100000,  10000,  1000,  100,  10,  2, 1.5, 1.3, 1.2, 1.1, 1.05),
#                 ncol=2)

#for (i in 1:nrow(limits))
#  print(c(limits[i,],"Mirror RR", mirrorRR(limits[i,],baseline)))

mirrorRR <- function(limits, reference, type="auto", fd=TRUE,
                     # types:  hundredths, tenths, quarters, halves, units, logscale, powers
                     cut=c(1.175, 1.4, 2, 3.5, 16, 50, 500),
                     # note that log(log(cut,base=10),base=10) is linear in (1:6)
                     base=NULL) {

  # Demonstrate log-log principle behind cutpoints of dynamic range
  # plot(log(log(cut,base=10),base=10))
  
  if (is.null(base)) base <- exp(1)
  
  limits <- sort(limits)
  mag <- limits[2]/limits[1]
  if (mag<=0) stop("powersAxis requires strictly positive axis range")
  minFact <- limits[1]/reference
  maxFact <- limits[2]/reference

  if ((type=="hundredths")||(mag<cut[1])) {  # Mark by 0.01x changes
    startFact <- ceiling(minFact*100)/100
    endFact <- floor(maxFact*100)/100
    atFact <- seq(startFact,endFact,by=0.01)
  }

  if ((type=="twentieths")||((mag>=cut[1])&&(mag<cut[2]))) {  # Mark by 0.05x changes
    startFact <- ceiling(minFact*20)/20
    endFact <- floor(maxFact*20)/20
    atFact <- seq(startFact,endFact,by=0.05)
  }
  
  if ((type=="tenths")||((mag>=cut[2])&&(mag<cut[3]))) {  # Mark by 0.1x changes
    startFact <- ceiling(minFact*10)/10
    endFact <- floor(maxFact*10)/10
    atFact <- seq(startFact,endFact,by=0.1)
  }
  
  if ((type=="quarters")||((mag>=cut[3])&&(mag<cut[4]))) {  # Mark by 0.25x changes
    startFact <- ceiling(minFact*4)/4
    endFact <- floor(maxFact*4)/4
    atFact <- seq(startFact,endFact,by=0.25)
  }

  if ((type=="halves")||((mag>=cut[4])&&(mag<cut[5]))) {  # Mark by 0.5x changes
    startFact <- ceiling(minFact*2)/2
    endFact <- floor(maxFact*2)/2
    atFact <- seq(startFact,endFact,by=0.5)
  }

  if ((type=="units")||((mag>=cut[5])&&(mag<cut[6]))) {  # Mark by 1x changes
    startFact <- ceiling(minFact)
    endFact <- floor(maxFact)
    atFact <- seq(startFact,endFact,by=1)
  }


  if ((type=="logscale")||((mag>=cut[6])&&(mag<cut[7]))) {  # Mark by 1x, 2x, 5x changes
    atFact <- 1
    done <- FALSE
    currbase <- 1
    while (!done) {
      cand <- currbase/c(2,5,10)
      validcand <- cand[cand>=minFact]
      atFact <- c(sort(validcand),atFact)
      if (length(validcand)<3) done <- TRUE
      currbase <- min(atFact)
    }

    done <- FALSE
    currbase <- 1
    while (!done) {
      cand <- currbase*c(2,5,10)
      validcand <- cand[cand<=maxFact]
      atFact <- c(atFact,sort(validcand))
      if (length(validcand)<3) done <- TRUE
      currbase <- max(atFact)
    }
  }

  if ((type=="powers")||(mag>=cut[7])) {  # Mark by 1x, 10x, 100x changes  
    startFact <- ceiling(log(minFact,base=10))
    endFact <- floor(log(maxFact,base=10))
    atFact <- 10^seq(startFact,endFact,by=1)
  }

  if (fd) {
    at <- reference*atFact - reference
  } else {
    at <- reference*atFact
  }
  labels <- paste(atFact,"x",sep="")
  list(at=at,labels=labels)
}

sigPch <- function(object, sig=16, nsig=1, rr=TRUE) {

  if (rr) {rr <- 1} else {rr <- 0} 
  object$lower <- as.matrix(object$lower)
  object$upper <- as.matrix(object$upper)
  dims <- dim(object$lower)
  pch <- matrix(NA, nrow=dims[1], ncol=dims[2])
  for (i in 1:dims[1]) {
    for (j in 1:dims[2]) {
      cond <- ( (object$lower[i,j]-rr)*(object$upper[i,j]-rr))>0
      if (cond) {pch[i,j] <- sig} else {pch[i,j] <- nsig}
    }
  }
  pch

}
